@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2010-2015 The pygit2 contributors
+# Copyright 2010-2017 The pygit2 contributors
 #
 # This file is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2,
@@ -32,6 +32,7 @@ from __future__ import absolute_import
 from string import hexdigits
 import sys, tarfile
 from time import time
+
 if sys.version_info[0] < 3:
     from cStringIO import StringIO
 else:
@@ -40,10 +41,11 @@ else:
 import six
 
 # Import from pygit2
-from _pygit2 import Repository as _Repository
+from _pygit2 import Repository as _Repository, init_file_backend
 from _pygit2 import Oid, GIT_OID_HEXSZ, GIT_OID_MINPREFIXLEN
 from _pygit2 import GIT_CHECKOUT_SAFE, GIT_CHECKOUT_RECREATE_MISSING, GIT_DIFF_NORMAL
 from _pygit2 import GIT_FILEMODE_LINK
+from _pygit2 import GIT_BRANCH_LOCAL, GIT_BRANCH_REMOTE, GIT_BRANCH_ALL
 from _pygit2 import Reference, Tree, Commit, Blob
 
 from .config import Config
@@ -56,23 +58,11 @@ from .utils import to_bytes, is_string
 from .submodule import Submodule
 
 
-class Repository(_Repository):
-
-    def __init__(self, path, *args, **kwargs):
-        if not isinstance(path, six.string_types):
-            path = path.decode('utf-8')
-        super(Repository, self).__init__(path, *args, **kwargs)
+class BaseRepository(_Repository):
+    def __init__(self, backend, *args, **kwargs):
+        super(BaseRepository, self).__init__(backend, *args, **kwargs)
         self._common_init()
 
-    @classmethod
-    def _from_c(cls, ptr, owned):
-        cptr = ffi.new('git_repository **')
-        cptr[0] = ptr
-        repo = cls.__new__(cls)
-        super(cls, repo)._from_c(bytes(ffi.buffer(cptr)[:]), owned)
-        repo._common_init()
-        return repo
-
     def _common_init(self):
         self.remotes = RemoteCollection(self)
 
@@ -236,29 +226,32 @@ class Repository(_Repository):
 
     def checkout(self, refname=None, **kwargs):
         """
-        Checkout the given reference using the given strategy, and update
-        the HEAD.
+        Checkout the given reference using the given strategy, and update the
+        HEAD.
         The reference may be a reference name or a Reference object.
         The default strategy is GIT_CHECKOUT_SAFE | GIT_CHECKOUT_RECREATE_MISSING.
 
-        To checkout from the HEAD, just pass 'HEAD'::
-
-          >>> checkout('HEAD')
-
-        This is identical to calling checkout_head().
-
         If no reference is given, checkout from the index.
 
-        Arguments:
+        Parameters:
 
-        :param str|Reference refname: The reference to checkout. After checkout,
-          the current branch will be switched to this one.
+        refname : str or Reference
+            The reference to checkout. After checkout, the current branch will
+            be switched to this one.
 
-        :param int strategy: A ``GIT_CHECKOUT_`` value. The default is
-          ``GIT_CHECKOUT_SAFE``.
+        strategy : int
+            A ``GIT_CHECKOUT_`` value. The default is ``GIT_CHECKOUT_SAFE``.
 
-        :param str directory: Alternative checkout path to workdir.
+        directory : str
+            Alternative checkout path to workdir.
 
+        Examples:
+
+        * To checkout from the HEAD, just pass 'HEAD'::
+
+            >>> checkout('HEAD')
+
+          This is identical to calling checkout_head().
         """
 
         # Case 1: Checkout index
@@ -279,11 +272,6 @@ class Repository(_Repository):
         oid = reference.resolve().target
         treeish = self[oid]
         self.checkout_tree(treeish, **kwargs)
-        head = self.lookup_reference('HEAD')
-        if head.type == C.GIT_REF_SYMBOLIC:
-            from_ = self.head.shorthand
-        else:
-            from_ = head.target.hex
 
         self.set_head(refname)
 
@@ -291,12 +279,13 @@ class Repository(_Repository):
     # Setting HEAD
     #
     def set_head(self, target):
-        """Set HEAD to point to the given target
+        """
+        Set HEAD to point to the given target.
 
-        Arguments:
+        Parameters:
 
         target
-            The new target for HEAD. Can be a string or Oid (to detach)
+            The new target for HEAD. Can be a string or Oid (to detach).
         """
 
         if isinstance(target, Oid):
@@ -322,8 +311,23 @@ class Repository(_Repository):
 
         Keyword arguments:
 
+        a
+            None, a str (that refers to an Object, see revparse_single()) or a
+            Reference object.
+            If None, b must be None, too. In this case the working directory is
+            compared with the index. Otherwise the referred object is compared to
+            'b'.
+
+        b
+            None, a str (that refers to an Object, see revparse_single()) or a
+            Reference object.
+            If None, the working directory is compared to 'a'. (except
+            'cached' is True, in which case the index is compared to 'a').
+            Otherwise the referred object is compared to 'a'
+
         cached
-            use staged changes instead of workdir
+            if 'b' is None, by default the working directory is compared to 'a'.
+            If 'cached' is set to True, the index/staging area is used for comparing.
 
         flag
             a GIT_DIFF_* constant
@@ -418,29 +422,36 @@ class Repository(_Repository):
     def blame(self, path, flags=None, min_match_characters=None,
               newest_commit=None, oldest_commit=None, min_line=None,
               max_line=None):
-        """Return a Blame object for a single file.
+        """
+        Return a Blame object for a single file.
 
-        Arguments:
+        Parameters:
 
         path
             Path to the file to blame.
+
         flags
             A GIT_BLAME_* constant.
+
         min_match_characters
             The number of alphanum chars that must be detected as moving/copying
             within a file for it to associate those lines with the parent commit.
+
         newest_commit
             The id of the newest commit to consider.
+
         oldest_commit
-          The id of the oldest commit to consider.
+            The id of the oldest commit to consider.
+
         min_line
             The first line in the file to blame.
+
         max_line
             The last line in the file to blame.
 
         Examples::
 
-            repo.blame('foo.c', flags=GIT_BLAME_TRACK_COPIES_SAME_FILE)");
+            repo.blame('foo.c', flags=GIT_BLAME_TRACK_COPIES_SAME_FILE)
         """
 
         options = ffi.new('git_blame_options *')
@@ -485,6 +496,7 @@ class Repository(_Repository):
     @staticmethod
     def _merge_options(favor):
         """Return a 'git_merge_opts *'"""
+
         def favor_to_enum(favor):
             if favor == 'normal':
                 return C.GIT_MERGE_FILE_FAVOR_NORMAL
@@ -531,26 +543,31 @@ class Repository(_Repository):
             theirs._to_c() if theirs is not None else (ffi.NULL, ffi.NULL))
 
         err = C.git_merge_file_from_index(
-                cmergeresult, self._repo,
-                cancestor, cours, ctheirs,
-                ffi.NULL);
+            cmergeresult, self._repo,
+            cancestor, cours, ctheirs,
+            ffi.NULL);
         check_error(err)
 
         ret = ffi.string(cmergeresult.ptr,
-                cmergeresult.len).decode('utf-8')
+                         cmergeresult.len).decode('utf-8')
         C.git_merge_file_result_free(cmergeresult)
 
         return ret
 
     def merge_commits(self, ours, theirs, favor='normal'):
-        """Merge two arbitrary commits
+        """
+        Merge two arbitrary commits.
 
-        Arguments:
+        Returns: an index with the result of the merge.
+
+        Parameters:
 
         ours
             The commit to take as "ours" or base.
+
         theirs
             The commit which will be merged into "ours"
+
         favor
             How to deal with file-level conflicts. Can be one of
 
@@ -559,13 +576,10 @@ class Repository(_Repository):
             * theirs. The "theirs" side of the conflict region is used.
             * union. Unique lines from each side will be used.
 
-            for all but NORMAL, the index will not record a conflict.
+            For all but NORMAL, the index will not record a conflict.
 
         Both "ours" and "theirs" can be any object which peels to a commit or the id
         (string or Oid) of an object which peels to a commit.
-
-        Returns an index with the result of the merge
-
         """
 
         ours_ptr = ffi.new('git_commit **')
@@ -591,27 +605,31 @@ class Repository(_Repository):
         return Index.from_c(self, cindex)
 
     def merge_trees(self, ancestor, ours, theirs, favor='normal'):
-        """Merge two trees
+        """
+        Merge two trees.
 
-        Arguments:
+        Returns: an Index that reflects the result of the merge.
+
+        Parameters:
 
         ancestor
-            The tree which is the common ancestor between 'ours' and 'theirs'
+            The tree which is the common ancestor between 'ours' and 'theirs'.
+
         ours
             The commit to take as "ours" or base.
+
         theirs
-            The commit which will be merged into "ours"
+            The commit which will be merged into "ours".
+
         favor
-            How to deal with file-level conflicts. Can be one of
+            How to deal with file-level conflicts. Can be one of:
 
             * normal (default). Conflicts will be preserved.
             * ours. The "ours" side of the conflict region is used.
             * theirs. The "theirs" side of the conflict region is used.
             * union. Unique lines from each side will be used.
 
-            for all but NORMAL, the index will not record a conflict.
-
-        Returns an Index that reflects the result of the merge.
+            For all but NORMAL, the index will not record a conflict.
         """
 
         ancestor_ptr = ffi.new('git_tree **')
@@ -649,34 +667,54 @@ class Repository(_Repository):
                  only_follow_first_parent=None,
                  show_commit_oid_as_fallback=None, abbreviated_size=None,
                  always_use_long_format=None, dirty_suffix=None):
-        """Describe a commit-ish or the current working tree.
+        """
+        Describe a commit-ish or the current working tree.
 
-        :param committish: Commit-ish object or object name to describe, or
-            `None` to describe the current working tree.
-        :type committish: `str`, :class:`~.Reference`, or :class:`~.Commit`
+        Returns: The description (str).
 
-        :param int max_candidates_tags: The number of candidate tags to
-            consider. Increasing above 10 will take slightly longer but may
-            produce a more accurate result. A value of 0 will cause only exact
-            matches to be output.
-        :param int describe_strategy: A GIT_DESCRIBE_* constant.
-        :param str pattern: Only consider tags matching the given `glob(7)`
-            pattern, excluding the "refs/tags/" prefix.
-        :param bool only_follow_first_parent: Follow only the first parent
-            commit upon seeing a merge commit.
-        :param bool show_commit_oid_as_fallback: Show uniquely abbreviated
-            commit object as fallback.
-        :param int abbreviated_size: The minimum number of hexadecimal digits
-            to show for abbreviated object names. A value of 0 will suppress
-            long format, only showing the closest tag.
-        :param bool always_use_long_format: Always output the long format (the
-            nearest tag, the number of commits, and the abbrevated commit name)
-            even when the committish matches a tag.
-        :param str dirty_suffix: A string to append if the working tree is
-            dirty.
+        Parameters:
 
-        :returns: The description.
-        :rtype: `str`
+        committish : `str`, :class:`~.Reference`, or :class:`~.Commit`
+            Commit-ish object or object name to describe, or `None` to describe
+            the current working tree.
+
+        max_candidates_tags : int
+            The number of candidate tags to consider. Increasing above 10 will
+            take slightly longer but may produce a more accurate result. A
+            value of 0 will cause only exact matches to be output.
+
+        describe_strategy : int
+            Can be one of:
+
+            * `GIT_DESCRIBE_DEFAULT` - Only match annotated tags. (This is
+              equivalent to setting this parameter to `None`.)
+            * `GIT_DESCRIBE_TAGS` - Match everything under refs/tags/
+              (includes lightweight tags).
+            * `GIT_DESCRIBE_ALL` - Match everything under refs/ (includes
+              branches).
+
+        pattern : str
+            Only consider tags matching the given `glob(7)` pattern, excluding
+            the "refs/tags/" prefix.
+
+        only_follow_first_parent : bool
+            Follow only the first parent commit upon seeing a merge commit.
+
+        show_commit_oid_as_fallback : bool
+            Show uniquely abbreviated commit object as fallback.
+
+        abbreviated_size : int
+            The minimum number of hexadecimal digits to show for abbreviated
+            object names. A value of 0 will suppress long format, only showing
+            the closest tag.
+
+        always_use_long_format : bool
+            Always output the long format (the nearest tag, the number of
+            commits, and the abbrevated commit name) even when the committish
+            matches a tag.
+
+        dirty_suffix : str
+            A string to append if the working tree is dirty.
 
         Example::
 
@@ -691,7 +729,11 @@ class Repository(_Repository):
         if describe_strategy is not None:
             options.describe_strategy = describe_strategy
         if pattern:
-            options.pattern = ffi.new('char[]', to_bytes(pattern))
+            # The returned pointer object has ownership on the allocated
+            # memory. Make sure it is kept alive until git_describe_commit() or
+            # git_describe_workdir() are called below.
+            pattern_char = ffi.new('char[]', to_bytes(pattern))
+            options.pattern = pattern_char
         if only_follow_first_parent is not None:
             options.only_follow_first_parent = only_follow_first_parent
         if show_commit_oid_as_fallback is not None:
@@ -737,11 +779,122 @@ class Repository(_Repository):
         finally:
             C.git_describe_result_free(result[0])
 
+    #
+    # Stash
+    #
+    def stash(self, stasher, message=None, keep_index=False,
+              include_untracked=False, include_ignored=False):
+        """
+        Save changes to the working directory to the stash.
+
+        Returns: The Oid of the stash merge commit (Oid).
+
+        Parameters:
+
+        stasher : Signature
+            The identity of the person doing the stashing.
+
+        message : str
+            An optional description of stashed state.
+
+        keep_index : bool
+            Leave changes already added to the index in the working directory.
+
+        include_untracked : bool
+            Also stash untracked files.
+
+        include_ignored : bool
+            Also stash ignored files.
+
+        Example::
+
+            >>> repo = pygit2.Repsitory('.')
+            >>> repo.stash(repo.default_signature(), 'WIP: stashing')
+        """
+
+        if message:
+            stash_msg = ffi.new('char[]', to_bytes(message))
+        else:
+            stash_msg = ffi.NULL
+
+        flags = 0
+        flags |= keep_index * C.GIT_STASH_KEEP_INDEX
+        flags |= include_untracked * C.GIT_STASH_INCLUDE_UNTRACKED
+        flags |= include_ignored * C.GIT_STASH_INCLUDE_IGNORED
+
+        stasher_cptr = ffi.new('git_signature **')
+        ffi.buffer(stasher_cptr)[:] = stasher._pointer[:]
+
+        coid = ffi.new('git_oid *')
+        err = C.git_stash_save(coid, self._repo, stasher_cptr[0], stash_msg, flags)
+        check_error(err)
+
+        return Oid(raw=bytes(ffi.buffer(coid)[:]))
+
+    @staticmethod
+    def _stash_args_to_options(reinstate_index=False, **kwargs):
+        stash_opts = ffi.new('git_stash_apply_options *')
+        check_error(C.git_stash_apply_init_options(stash_opts, 1))
+
+        flags = reinstate_index * C.GIT_STASH_APPLY_REINSTATE_INDEX
+        stash_opts.flags = flags
+
+        copts, refs = Repository._checkout_args_to_options(**kwargs)
+        stash_opts.checkout_options = copts[0]
+
+        return stash_opts
+
+    def stash_apply(self, index=0, **kwargs):
+        """
+        Apply a stashed state in the stash list to the working directory.
+
+        Parameters:
+
+        index : int
+            The position within the stash list of the stash to apply. 0 is the
+            most recent stash.
+
+        reinstate_index : bool
+            Try to reinstate stashed changes to the index.
+
+        The checkout options may be customized using the same arguments taken by
+        Repository.checkout().
+
+        Example::
+
+            >>> repo = pygit2.Repsitory('.')
+            >>> repo.stash(repo.default_signature(), 'WIP: stashing')
+            >>> repo.stash_apply(strategy=GIT_CHECKOUT_ALLOW_CONFLICTS)
+        """
+        stash_opts = Repository._stash_args_to_options(**kwargs)
+        check_error(C.git_stash_apply(self._repo, index, stash_opts))
+
+    def stash_drop(self, index=0):
+        """
+        Remove a stashed state from the stash list.
+
+        Parameters:
+
+        index : int
+            The position within the stash list of the stash to remove. 0 is
+            the most recent stash.
+        """
+        check_error(C.git_stash_drop(self._repo, index))
+
+    def stash_pop(self, index=0, **kwargs):
+        """Apply a stashed state and remove it from the stash list.
+
+        For arguments, see Repository.stash_apply().
+        """
+        stash_opts = Repository._stash_args_to_options(**kwargs)
+        check_error(C.git_stash_pop(self._repo, index, stash_opts))
+
     #
     # Utility for writing a tree into an archive
     #
     def write_archive(self, treeish, archive, timestamp=None, prefix=''):
-        """Write treeish into an archive
+        """
+        Write treeish into an archive.
 
         If no timestamp is provided and 'treeish' is a commit, its committer
         timestamp will be used. Otherwise the current time will be used.
@@ -749,14 +902,17 @@ class Repository(_Repository):
         All path names in the archive are added to 'prefix', which defaults to
         an empty string.
 
-        Arguments:
+        Parameters:
 
         treeish
             The treeish to write.
+
         archive
-            An archive from the 'tarfile' module
+            An archive from the 'tarfile' module.
+
         timestamp
             Timestamp to use for the files in the archive.
+
         prefix
             Extra prefix to add to the path names in the archive.
 
@@ -765,7 +921,7 @@ class Repository(_Repository):
             >>> import tarfile, pygit2
             >>>> with tarfile.open('foo.tar', 'w') as archive:
             >>>>     repo = pygit2.Repsitory('.')
-            >>>>     repo.write_archive(archive, repo.head.target)
+            >>>>     repo.write_archive(repo.head.target, archive)
         """
 
         # Try to get a tree form whatever we got
@@ -797,11 +953,11 @@ class Repository(_Repository):
             info = tarfile.TarInfo(prefix + entry.path)
             info.size = len(content)
             info.mtime = timestamp
-            info.uname = info.gname = 'root' # just because git does this
+            info.uname = info.gname = 'root'  # just because git does this
             if entry.mode == GIT_FILEMODE_LINK:
-                info.type = archive.SYMTYPE
-                info.linkname = content
-                info.mode = 0o777 # symlinks get placeholder
+                info.type = tarfile.SYMTYPE
+                info.linkname = content.decode("utf-8")
+                info.mode = 0o777  # symlinks get placeholder
                 info.size = 0
                 archive.addfile(info)
             else:
@@ -811,22 +967,23 @@ class Repository(_Repository):
     # Ahead-behind, which mostly lives on its own namespace
     #
     def ahead_behind(self, local, upstream):
-        """Calculate how many different commits are in the non-common parts
-        of the history between the two given ids.
+        """
+        Calculate how many different commits are in the non-common parts of the
+        history between the two given ids.
 
-        Ahead is how many commits are in the ancestry of the 'local'
-        commit which are not in the 'upstream' commit. Behind is the
-        opposite.
+        Ahead is how many commits are in the ancestry of the 'local' commit
+        which are not in the 'upstream' commit. Behind is the opposite.
 
-        Arguments
+        Returns: a tuple of two integers with the number of commits ahead and
+        behind respectively.
+
+        Parameters:
 
         local
-            The commit which is considered the local or current state
-        upstream
-            The commit which is considered the upstream
+            The commit which is considered the local or current state.
 
-        Returns a tuple of two integers with the number of commits ahead and
-        behind respectively.
+        upstream
+            The commit which is considered the upstream.
         """
 
         if not isinstance(local, Oid):
@@ -848,21 +1005,24 @@ class Repository(_Repository):
     # Git attributes
     #
     def get_attr(self, path, name, flags=0):
-        """Retrieve an attribute for a file by path
+        """
+        Retrieve an attribute for a file by path.
 
-        Arguments
+        Returns: a boolean, None if the value is unspecified, or string with
+        the value of the attribute.
+
+        Parameters:
 
         path
             The path of the file to look up attributes for, relative to the
-            workdir root
+            workdir root.
+
         name
-            The name of the attribute to look up
+            The name of the attribute to look up.
+
         flags
             A combination of GIT_ATTR_CHECK_ flags which determine the
-            lookup order
-
-        Returns either a boolean, None (if the value is unspecified) or string
-        with the value of the attribute.
+            lookup order.
         """
 
         cvalue = ffi.new('char **')
@@ -905,3 +1065,133 @@ class Repository(_Repository):
 
         err = C.git_repository_set_ident(self._repo, to_bytes(name), to_bytes(email))
         check_error(err)
+
+    def revert_commit(self, revert_commit, our_commit, mainline=0):
+        """
+        Reverts the given Commit against the given "our" Commit, producing an
+        Index that reflects the result of the revert.
+
+        Returns: an Index with the result of the revert.
+
+        Parameters:
+
+        revert_commit
+            The Commit to revert.
+
+        our_commit
+            The Commit to revert against (eg, HEAD).
+
+        mainline
+            The parent of the revert Commit, if it is a merge (i.e. 1, 2).
+        """
+        cindex = ffi.new('git_index **')
+        revert_commit_ptr = ffi.new('git_commit **')
+        our_commit_ptr = ffi.new('git_commit **')
+
+        ffi.buffer(revert_commit_ptr)[:] = revert_commit._pointer[:]
+        ffi.buffer(our_commit_ptr)[:] = our_commit._pointer[:]
+
+        opts = ffi.new('git_merge_options *')
+        err = C.git_merge_init_options(opts, C.GIT_MERGE_OPTIONS_VERSION)
+        check_error(err)
+
+        err = C.git_revert_commit(
+            cindex, self._repo, revert_commit_ptr[0], our_commit_ptr[0], mainline, opts
+        )
+        check_error(err)
+
+        return Index.from_c(self, cindex)
+
+
+class Branches(object):
+    def __init__(self, repository, flag=GIT_BRANCH_ALL):
+        self._repository = repository
+        self._flag = flag
+
+        if flag == GIT_BRANCH_ALL:
+            self.local = Branches(repository, flag=GIT_BRANCH_LOCAL)
+            self.remote = Branches(repository, flag=GIT_BRANCH_REMOTE)
+
+    def __getitem__(self, name):
+        branch = None
+        if self._flag & GIT_BRANCH_LOCAL:
+            branch = self._repository.lookup_branch(name, GIT_BRANCH_LOCAL)
+
+        if branch is None and self._flag & GIT_BRANCH_REMOTE:
+            branch = self._repository.lookup_branch(name, GIT_BRANCH_REMOTE)
+
+        if branch is None:
+            raise KeyError('Branch not found: {}'.format(name))
+
+        return branch
+
+    def get(self, key):
+        try:
+            return self[key]
+        except KeyError:
+            return None
+
+    def __iter__(self):
+        for branch_name in self._repository.listall_branches(self._flag):
+            yield branch_name
+
+    def create(self, name, commit, force=False):
+        return self._repository.create_branch(name, commit, force)
+
+    def delete(self, name):
+        self[name].delete()
+
+    def __contains__(self, name):
+        return self.get(name) is not None
+
+
+class References(object):
+    def __init__(self, repository):
+        self._repository = repository
+
+    def __getitem__(self, name):
+        return self._repository.lookup_reference(name)
+
+    def get(self, key):
+        try:
+            return self[key]
+        except KeyError:
+            return None
+
+    def __iter__(self):
+        for ref_name in self._repository.listall_references():
+            yield ref_name
+
+    def create(self, name, target, force=False):
+        return self._repository.create_reference(name, target, force)
+
+    def delete(self, name):
+        self[name].delete()
+
+    def __contains__(self, name):
+        return self.get(name) is not None
+
+    @property
+    def objects(self):
+        return self._repository.listall_reference_objects()
+
+
+class Repository(BaseRepository):
+    def __init__(self, path, *args, **kwargs):
+        if not isinstance(path, six.string_types):
+            path = path.decode('utf-8')
+
+        path_backend = init_file_backend(path)
+        super(Repository, self).__init__(backend=path_backend, *args, **kwargs)
+
+        self.branches = Branches(self)
+        self.references = References(self)
+
+    @classmethod
+    def _from_c(cls, ptr, owned):
+        cptr = ffi.new('git_repository **')
+        cptr[0] = ptr
+        repo = cls.__new__(cls)
+        super(cls, repo)._from_c(bytes(ffi.buffer(cptr)[:]), owned)
+        repo._common_init()
+        return repo
